<?php

require_once (drupal_get_path('module', 'reptag').'/reptag_ajax.inc');

/**
 * Implementation of _reptag_manage_admin().
 *  - general settings - 
 */
function _reptag_manage_admin() {

  $form['global'] = array(
    '#type' => 'fieldset',
    '#title' => t('RepTag - Settings'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE
  );
  $form['global']['reptag_enable'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable Rep[lacement]Tags'),
    '#return_value' => 1,
    '#default_value' => variable_get('reptag_enable', 0)
  );
  $form['global']['reptag_preview'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable Rep[lacement]Tags in Preview'),
    '#return_value' => 1,
    '#default_value' => variable_get('reptag_preview', 1)
  );
  $form['global']['reptag_showhelp'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show \'Rep[lacement]Tags - Help\' section on add/edit node pages'),
    '#return_value' => 1,
    '#default_value' => variable_get('reptag_showhelp', 1)
  );
  $form['global']['reptag_tableprio'] = array(
    '#type' => 'checkbox',
    '#title' => t('Privilege SiteWide tags over User tags'),
    '#return_value' => 1,
    '#default_value' => variable_get('reptag_tableprio', 0),
    '#description' => t('By default users can override site wide tags with their own tags')
  );
  
  //node-types
  $node_types = node_get_types();
  if (!empty($node_types)) {
    $form['global']['node_types'] = array(
      '#type' => 'fieldset',
      '#title' => t('Node - Types'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#tree' => TRUE,
      '#weight' => '1'
    );
    foreach ($node_types as $type => $label) {
      $form['global']['node_types'][$type] = array(
        '#type' => 'checkbox',
        '#title' => t('Disable RepTags for ').'<strong>'.strtoupper($label).'</strong>',
        '#return_value' => 1,
        '#default_value' => variable_get('reptag_type_disable_'.$type, 0)
      );
    }
    if (module_exist('comment')) {
      $form['global']['node_types']['comments'] = array(
        '#type' => 'checkbox',
        '#title' => t('Disable RepTags for ').'<strong>'.strtoupper(t('comments')).'</strong>',
        '#return_value' => 1,
        '#default_value' => variable_get('reptag_type_disable_comments', 1)
      );
    }
  }
  
  //expert - settings
  $form['global']['expert'] = array(
    '#type' => 'fieldset',
    '#title' => t('Expert - Settings'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#tree' => TRUE,
    '#weight' => '2'
  );
  $form['global']['expert']['reptag_validate'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable Preview - Validation'),
    '#return_value' => 1,
    '#default_value' => variable_get('reptag_validate', 1),
    '#description' => t('Run html-filter on Preview to find invalid tags.')
  );
  $form['global']['expert']['reptag_cache'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable Partial PageCache'),
    '#return_value' => 1,
    '#default_value' => variable_get('reptag_cache', 0),
    '#description' => t('Expand static tags on form submission and cache the result. Only dynamic tags are processed on page view.<br />To disable dynamic tags completely check \'Disable Dynamic Replacements\'.')
  );
  $form['global']['expert']['reptag_staticonly'] = array(
    '#type' => 'checkbox',
    '#title' => t('Disable Dynamic Replacements'),
    '#return_value' => 1,
    '#default_value' => variable_get('reptag_staticonly', 0),
    '#description' => t('Run reptag on form submission only (disables certain modules, e.g. System, Block, ...).<br />Modules affected by this option are marked \'Static\' = \'No\' below.'),
    '#attributes' => (variable_get('reptag_cache', 0) ? array() : array('disabled' => '' ))
  );
  $form['global']['expert']['reptag_xajax_path'] = array(
    '#type' => 'textfield',
    '#title' => t('XAJAX-Path'),
    '#default_value' => variable_get('reptag_xajax_path', ''),
    '#size' => 50,
    '#maxlength' => 255,
    '#description' => t('Path to \'xajax.inc.php\' (e.g. modules/reptag/xajax/xajax.inc.php).')
  );
  $form['global']['expert']['reptag_debug'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable Debugging'),
    '#return_value' => 1,
    '#default_value' => variable_get('reptag_debug', 0),
    '#description' => t('Print debug listings (print_r) of module variables.')
  );
  $form['modules'] = array(
    '#type' => 'fieldset',
    '#title' => t('Modules'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#tree' => TRUE
  );
  
  //modules
  $files = file_scan_directory(drupal_get_path('module', 'reptag'), '\.tags$');
  $form['modules']['table'] = array('#theme' => 'reptag_modules_table');
  foreach ($files as $filename => $file) {
    include_once $file->filename;
    $static_module = is_array($reptags_info[$file->name]) ? $reptags_info[$file->name][1] : FALSE;
    $disable_module = variable_get('reptag_cache', 0) && variable_get('reptag_staticonly', 0) && !$static_module;
    $form['modules']['table'][$file->name]['enabled'] = array(
      '#type' => 'checkbox',
      '#title' => '',
      '#return_value' => 1,
      '#default_value' => $disable_module ? FALSE : variable_get('reptag_module_'.$file->name, 0),
      '#prefix' => '<div align="center">',
      '#suffix' => '</div>',
      '#attributes' => ($disable_module ? array('disabled' => '' ) : array())
    );
    $form['modules']['table'][$file->name]['module'] = array(
      '#value' => $file->basename,
      '#prefix' => '<strong>',
      '#suffix' => '</strong>'
    );
    $form['modules']['table'][$file->name]['info'] = array(
      '#value' => is_array($reptags_info[$file->name]) ?
        $reptags_info[$file->name][0] : $reptags_info[$file->name],
      '#prefix' => '<i>',
      '#suffix' => '</i>'
    );
    $form['modules']['table'][$file->name]['static'] = array(
      '#value' => $static_module ? t('Yes') : t('No'),
      '#prefix' => '<div align="center">',
      '#suffix' => '</div>'
    );
  }
  
  $form['roles'] = array(
    '#type' => 'fieldset',
    '#title' => t('Permission - Roles'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#tree' => TRUE
  );
  
  $roles = _reptag_roles_load();
  $files = array_values(file_scan_directory(drupal_get_path('module', 'reptag'), '\.tags$'));
  
  $form['roles']['table'] = array('#theme' => 'reptag_roles_table');
  $form['roles']['table']['header'] = array('#data' => array_merge(array(t('Option / Module')), array_keys($roles)));
  
  //module permissions
  foreach($files as $file) {
    $form['roles']['table'][$file->name]['label'] = array(
      '#value' => t('Module:').' <strong>'.$file->basename.'</strong>'
    );
    foreach ($roles as $rid) {
      $form['roles']['table'][$file->name][$rid] = array(
        '#type' => 'checkbox',
        '#title' => '',
        '#return_value' => 1,
        '#default_value' => in_array($rid, (array)unserialize(variable_get('reptag_module_'.$file->name.'_roles', ''))),
        '#prefix' => '<div align="center">',
        '#suffix' => '</div>'
      );
    }    
  }
  //spacer
  $form['roles']['table']['spacer']['label'] = array(
    '#value' => ''
  );
  foreach ($roles as $rid) {
    $form['roles']['table']['spacer'][$rid] = array(
      '#value' => ''
    );
  }
  //additional permissions
  $form['roles']['table']['plainrep']['label'] = array(
    '#value' => t('Plain text replacements only'),
    '#prefix' => '<strong>',
    '#suffix' => '</strong>'
  );
  foreach ($roles as $rid) {
    $form['roles']['table']['plainrep'][$rid] = array(
      '#type' => 'checkbox',
      '#title' => '',
      '#return_value' => 1,
      '#default_value' => in_array($rid, (array)unserialize(variable_get('reptag_module_plainrep_roles', ''))),
      '#prefix' => '<div align="center">',
      '#suffix' => '</div>'
    );
  }
  
  //exclude tags
  if ($ajax = _reptag_ajax_site_init()) {
    //ajax
    
    $form['roles']['exclude'] = array(
      '#type' => 'fieldset',
      '#title' => t('Exclude Tags'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#tree' => TRUE,
      '#prefix' => '<br />'
    );
    $form['roles']['exclude']['table'] = array('#theme' => 'reptag_exclude'); 
    foreach ($roles as $role => $rid) {
      $form['roles']['exclude']['table'][$rid]['role'] = array(
        '#value' => $role
      );
      $form['roles']['exclude']['table'][$rid]['tags'] = array(
        '#type' => 'textfield',
        '#default_value' => _reptag_exclude_loadtags_string($rid),
        '#size' => 60,
        '#maxlength' => 1024,
        '#attributes' => array('disabled' => 'disabled')
      );
      $form['roles']['exclude']['table'][$rid]['add'] = array(
        '#type' => 'reptag_button',
        '#value' => t('Append'),
        '#attributes' => array(
          'id' => 'edit-roles-exclude-table-'.$rid.'-add',
          'onclick' => 'xajax__reptag_ajax_exclude(1, '.$rid.');'),
        '#button_type' => 'reptag_button'
      );
      $form['roles']['exclude']['table'][$rid]['remove'] = array(
        '#type' => 'reptag_button',
        '#value' => t('Remove'),
        '#attributes' => array(
          'id' => 'edit-roles-exclude-table-'.$rid.'-remove',
          'onclick' => 'xajax__reptag_ajax_exclude(0, '.$rid.');'),
        '#button_type' => 'reptag_button'
      );
    }
    $form['roles']['exclude']['container']['add'] = array('#theme' => 'reptag_exclude_add'); 
    $form['roles']['exclude']['container']['add']['value'] = array(
      '#type' => 'textfield',
      '#title' => '',
      '#default_value' => '',
      '#size' => 35,
      '#maxlength' => 128,
      '#autocomplete_path' => 'reptag/autocomplete'
    );
    $form['roles']['exclude']['container']['add']['button'] = array(
      '#type' => 'reptag_button',
      '#value' => t('Add'),
      '#attributes' => array(
        'id' => 'edit-roles-exclude-container-addbtn',
        'onclick' => 'xajax__reptag_ajax_exclude_add(reptag_exclude_getrid(), reptag_exclude_add_value());'),
      '#button_type' => 'reptag_button'
    );
    
  } else {
    //classic
    
    $form['roles']['exclude'] = array(
      '#type' => 'fieldset',
      '#title' => t('Exclude Tags'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#tree' => TRUE,
      '#prefix' => '<br />'
    );
    reset($roles);
    foreach ($roles as $role => $rid) {
      $form['roles']['exclude']['table'][$rid] = array(
        '#type' => 'textfield',
        '#title' => t('Exclude for ').$role,
        '#default_value' => _reptag_exclude_loadtags_string($rid),
        '#size' => 80,
        '#maxlength' => 1024,
        '#description' => t('Tags to exclude for ').$role.t(' seperated by \'|\', e.g. \'$MYTAG$\' or \'#{BACK}(.*?){/BACK}#s\'.')
      );
    }
  }
  
  $form['buttons']['submit'] = array('#type' => 'submit', '#value' => t('Save configuration'));
  $form['buttons']['reset'] = array('#type' => 'submit', '#value' => t('Reset to defaults'));
  return drupal_get_form('_reptag_manage_admin', $form);
}

/**
 * Implementation of _reptag_manage_site().
 *  - manage site-wide tags - 
 */
function _reptag_manage_site() {
  
  $ajax = _reptag_ajax_site_init();
  $reptag_table = _reptag_table_load(array(0), 2);

  if (variable_get('reptag_debug', 0)) {
    drupal_set_message(print_r($reptag_table, true), 'error');
  }
  
  $form['admin'] = array(
    '#type' => 'fieldset',
    '#title' => t('SiteWide - Rep[lacement]Tags'),
    '#collapsible' => FALSE,
    '#collapsed' => FALSE,
    '#tree' => TRUE
  );
  $form['admin']['table'] = array('#theme' => 'reptag_table'); 
  $form['admin']['table']['sample']['tag'] = array(
    '#value' => t('<strong>MYTAG</strong>')
  );
  $form['admin']['table']['sample']['replacement'] = array(
    '#value' => t('<strong>Hello, I\'m Mr. Repl Text.</strong>')
  );
  $num = 0;
  foreach ($reptag_table as $name => $value) {
    $form['admin']['table'][$num]['tag'] = array(
      '#type' => 'textfield',
      '#default_value' => $name,
      '#size' => 20,
      '#maxlength' => 255
    );
    $form['admin']['table'][$num]['arrow'] = array(
      '#value' => '=>',
      '#prefix' => '<div id="edit-admin-table-'.$num.'-arrow">',
      '#suffix' => '</div>'
    );
    $form['admin']['table'][$num]['replacement'] = array(
      '#type' => 'textfield',
      '#default_value' => $value,
      '#size' => 65,
      '#maxlength' => 1024
    );
    if ($ajax) {
      $form['admin']['table'][$num]['edit'] = array(
        '#type' => 'reptag_button',
        '#value' => t('Edit'),
        '#attributes' => array(
          'id' => 'edit-admin-table-'.$num.'-edit',
          'onclick' => 'xajax__reptag_ajax_tagedit('.$num.', reptag_admin_name('.$num.'), reptag_admin_value('.$num.'), 0);'),
        '#button_type' => 'reptag_button'
      );
      $form['admin']['table'][$num]['remove'] = array(
        '#type' => 'reptag_button',
        '#value' => t('Del'),
        '#attributes' => array(
          'id' => 'edit-admin-table-'.$num.'-remove',
          'onclick' => 'xajax__reptag_ajax_tagremove('.$num.', 0);'),
        '#button_type' => 'reptag_button'
      );
    }
    $num++;
  }
  $form['admin']['table'][$num]['tag'] = array(
    '#type' => 'textfield',
    '#default_value' => '',
    '#size' => 20,
    '#maxlength' => 255
  );
  $form['admin']['table'][$num]['arrow'] = array(
    '#value' => '=>',
    '#prefix' => '<div id="edit-admin-table-'.$num.'-arrow">',
    '#suffix' => '</div>'
  );
  $form['admin']['table'][$num]['replacement'] = array(
    '#type' => 'textfield',
    '#default_value' => '',
    '#size' => 65,
    '#maxlength' => 1024
  );
  if ($ajax) {
    $form['admin']['table'][$num]['add'] = array(
      '#type' => 'reptag_button',
      '#value' => t('Add'),
      '#attributes' => array(
        'id' => 'edit-admin-table-'.$num.'-add',
        'onclick' => 'xajax__reptag_ajax_tagadd(reptag_admin_name('.$num.'), reptag_admin_value('.$num.'), 0);'),
      '#button_type' => 'reptag_button'
    );
  }

  if (!$ajax) {
    $form['buttons']['submit'] = array('#type' => 'submit', '#value' => t('Save configuration'));
    $form['buttons']['reset'] = array('#type' => 'submit', '#value' => t('Reset to defaults'));
  }
  
  return drupal_get_form('_reptag_manage_site', $form);
}

/**
 * Implementation of _reptag_manage_user().
 *  - manage user tags -
 */
function _reptag_manage_user() {

  global $user;
  $ajax = _reptag_ajax_site_init();
  $reptag_table = _reptag_table_load(array($user->uid), 2);

  if (variable_get('reptag_debug', 0)) {
    drupal_set_message(print_r($reptag_table, true), 'error');
  }
  
  $form['user'] = array(
    '#type' => 'fieldset',
    '#title' => t('User - Rep[lacement]Tags'),
    '#collapsible' => FALSE,
    '#collapsed' => FALSE,
    '#tree' => TRUE
  );
  $form['user']['table'] = array('#theme' => 'reptag_table');
  $form['user']['table']['sample']['tag'] = array(
    '#value' => t('<strong>MYTAG</strong>')
  );
  $form['user']['table']['sample']['replacement'] = array(
    '#value' => t('<strong>Hello, I\'m Mr. Repl Text.</strong>')
  );    
  $num = 0;
  foreach ($reptag_table as $name => $value) {
    $form['user']['table'][$num]['tag'] = array(
      '#type' => 'textfield',
      '#default_value' => $name,
      '#size' => 20,
      '#maxlength' => 255
    );
    $form['user']['table'][$num]['arrow'] = array(
      '#value' => '=>',
      '#prefix' => '<div id="edit-user-table-'.$num.'-arrow">',
      '#suffix' => '</div>'
    );
    $form['user']['table'][$num]['replacement'] = array(
      '#type' => 'textfield',
      '#default_value' => $value,
      '#size' => 65,
      '#maxlength' => 1024
    );
    if ($ajax) {
      $form['user']['table'][$num]['edit'] = array(
        '#type' => 'reptag_button',
        '#value' => t('Edit'),
        '#attributes' => array(
          'id' => 'edit-user-table-'.$num.'-edit',
          'onclick' => 'xajax__reptag_ajax_tagedit('.$num.', reptag_user_name('.$num.'), reptag_user_value('.$num.'), '.$user->uid.');'),
        '#button_type' => 'reptag_button'
      );
      $form['user']['table'][$num]['remove'] = array(
        '#type' => 'reptag_button',
        '#value' => t('Del'),
        '#attributes' => array(
          'id' => 'edit-user-table-'.$num.'-remove',
          'onclick' => 'xajax__reptag_ajax_tagremove('.$num.', '.$user->uid.');'),
        '#button_type' => 'reptag_button'
      );
    }
    $num++;
  }
  $form['user']['table'][$num]['tag'] = array(
    '#type' => 'textfield',
    '#default_value' => '',
    '#size' => 20,
    '#maxlength' => 255
  );
  $form['user']['table'][$num]['arrow'] = array(
    '#value' => '=>',
    '#prefix' => '<div id="edit-user-table-'.$num.'-arrow">',
    '#suffix' => '</div>'
  );
  $form['user']['table'][$num]['replacement'] = array(
    '#type' => 'textfield',
    '#default_value' => '',
    '#size' => 65,
    '#maxlength' => 1024
  );
  if ($ajax) {
    $form['user']['table'][$num]['add'] = array(
      '#type' => 'reptag_button',
      '#value' => t('Add'),
      '#attributes' => array(
        'id' => 'edit-user-table-'.$num.'-add',
        'onclick' => 'xajax__reptag_ajax_tagadd(reptag_user_name('.$num.'), reptag_user_value('.$num.'), '.$user->uid.');'),
      '#button_type' => 'reptag_button'
    );
  } 

  if (!$ajax) {
    $form['buttons']['submit'] = array('#type' => 'submit', '#value' => t('Save configuration'));
    $form['buttons']['reset'] = array('#type' => 'submit', '#value' => t('Reset to defaults'));
  }
  
  return drupal_get_form('_reptag_manage_user', $form);
}

/**
 * Implementation of _submit().
 * - for _reptag_manage_admin -
 */
function _reptag_manage_admin_submit($form_id, $form_data) {

  if ($_POST['op'] == t('Save configuration')) {
  
    variable_set('reptag_enable', $form_data['reptag_enable']);
    variable_set('reptag_preview', $form_data['reptag_preview']);
    variable_set('reptag_showhelp', $form_data['reptag_showhelp']);
    variable_set('reptag_tableprio', $form_data['reptag_tableprio']);
    variable_set('reptag_validate', $form_data['expert']['reptag_validate']);
    variable_set('reptag_cache', $form_data['expert']['reptag_cache']);
    variable_set('reptag_staticonly', $form_data['expert']['reptag_staticonly']);
    variable_set('reptag_xajax_path', $form_data['expert']['reptag_xajax_path']);
    variable_set('reptag_debug', $form_data['expert']['reptag_debug']);
    
    //node types
    foreach ($form_data['node_types'] as $type => $value) {
      variable_set('reptag_type_disable_'.$type, $value);
    }
    
    //modules
    foreach ($form_data['modules']['table'] as $module => $value) {
      variable_set('reptag_module_'.$module, $value['enabled']);
      //check prerequisites
      if ($value['enabled'] && function_exists('_'.$module.'_require')) {
        $require_func = '_'.$module.'_require';
        if (!$require_func()) {
          drupal_set_message(strtoupper($module).t(' module has been disabled!<br />It does not comply with all requirements (e.g. include dependencies, PEAR packages, ...).'), 'error');
          variable_set('reptag_module_'.$module, 0);      
        }
      }
    }
    
    //roles
    foreach ($form_data['roles']['table'] as $module => $role) {
      $roles = array();
      foreach ($role as $rid => $value) {
        if ($value) {
          $roles[] = $rid;
        }
      }
      variable_set('reptag_module_'.$module.'_roles', serialize($roles));
    }
    
    //exclude
    foreach ($form_data['roles']['exclude']['table'] as $rid => $data) {
      if (is_array($data)) {
        _reptag_exclude_storetags_string($rid, $data['tags']);
      } else {
        _reptag_exclude_storetags_string($rid, $data);
      }
    }
    
  } else {
  
    //TODO: add confirmation page for reset
  
    variable_set('reptag_enable', 1);
    variable_set('reptag_preview', 1);
    variable_set('reptag_showhelp', 1);
    variable_set('reptag_tableprio', 0);
    variable_set('reptag_validate', 1);
    variable_set('reptag_cache', 0);
    variable_set('reptag_staticonly', 0);
    variable_set('reptag_xajax_path', '');
    variable_set('reptag_debug', 0);
    
  }
  
  drupal_set_message(t('The configuration options have been saved.'));
  if (variable_get('reptag_validate', 1) && !class_exists('DOMDocument') && !function_exists('xml_parser_create')) {
    drupal_set_message(t('You dont have neither \'DOMDocument\' class nor \'xml_parser_create\' function available.<br />HTML-Validation is not possible and will be disabled!'), 'error');
    variable_set('reptag_validate', 0);
  }
  drupal_goto('admin/settings/reptag/admin');
}

/**
 * Implementation of _submit().
 * - for _reptag_manage_site -
 */
function _reptag_manage_site_submit($form_id, $form_data) {

  if ($_POST['op'] == t('Save configuration')) {
    
    //sitewide
    _reptag_table_clear(0);
    foreach ($form_data['admin']['table'] as $reptag) {
      $tag = $reptag['tag'];
      $replacement = $reptag['replacement'];
      if (!empty($tag) && !empty($replacement)) {
        $tag = '#\$'.strtoupper($tag).'\$#si'; 
        _reptag_table_set($tag, $replacement, 0);
      }
    }
    
  } else {
  
    //TODO: add confirmation page for reset
  
    //reset
    db_lock_table('reptag_vars');
    db_query("DELETE FROM {reptag_vars} WHERE uid = '0'");
    db_unlock_tables();
    
  }
  
  drupal_set_message(t('The configuration options have been saved.'));
  drupal_goto('admin/settings/reptag/site');
}

/**
 * Implementation of _submit().
 * - for _reptag_manage_user -
 */
function _reptag_manage_user_submit($form_id, $form_data) {

  global $user;
  
  if ($_POST['op'] == t('Save configuration')) {
    
    //user
    _reptag_table_clear($user->uid);
    foreach ($form_data['user']['table'] as $reptag) {
      $tag = $reptag['tag'];
      $replacement = $reptag['replacement'];
      if (!empty($tag) && !empty($replacement)) {
        $tag = '#\$'.strtoupper($tag).'\$#si'; 
        _reptag_table_set($tag, $replacement, $user->uid);
      }
    }
    
  } else {
  
    //TODO: add confirmation page for reset
  
    //reset
    db_lock_table('reptag_vars');
    db_query("DELETE FROM {reptag_vars} WHERE uid = '%s'", $user->uid);
    db_unlock_tables();
  
  }
  
  drupal_set_message(t('The configuration options have been saved.'));
  drupal_goto('admin/settings/reptag/user');
}

/**
 * Implementation of _submit().
 *  - for reptag_workspace
 */
function reptag_workspace_submit($form_id, $form_data) {

  global $user;
  
  if ($_POST['op'] == t('Save configuration')) {
    
    //user
    _reptag_table_clear($user->uid);
    foreach ($form_data['user']['table'] as $reptag) {
      $tag = $reptag['tag'];
      $replacement = $reptag['replacement'];
      if (!empty($tag) && !empty($replacement)) {
        $tag = '#\$'.strtoupper($tag).'\$#si'; 
        _reptag_table_set($tag, $replacement, $user->uid);
      }
    }
    
  }
  
  drupal_set_message(t('The configuration options have been saved.'));
  drupal_goto('workspace/reptag');
}

/**
 * Implementation of theme_reptag_table().
 *  - theming for reptag_table
 */
function theme_reptag_table($form) {
  foreach (element_children($form) as $key) {
    $row = array();
    $row[] = form_render($form[$key]['tag']);
    $row[] = form_render($form[$key]['arrow']);
    $row[] = form_render($form[$key]['replacement']);
    if (isset($form[$key]['add'])) {
      $row[] = form_render($form[$key]['add']);
    } else {
      $row[] = form_render($form[$key]['edit']);
    }
    $row[] = form_render($form[$key]['remove']);
    $rows[] = $row;
  }
  $header = array(t('Tag'), '', t('Replacement'));
  return theme_table($header, $rows); 
}

/**
 * Implementation of theme_reptag_modules_table().
 *  - theming for reptag_modules_table
 */
function theme_reptag_modules_table($form) {
  foreach (element_children($form) as $key) {
    $row = array();
    $row[] = form_render($form[$key]['enabled']);
    $row[] = form_render($form[$key]['module']);
    $row[] = form_render($form[$key]['info']);
    $row[] = form_render($form[$key]['static']);
    $rows[] = $row;
  }
  $header = array(t('Enabled'), t('Module'), t('Info'), t('Static'));
  return theme_table($header, $rows);
}

/**
 * Implementation of theme_reptag_roles_table().
 *  - theming for reptag_roles_table
 */
function theme_reptag_roles_table($form) {
  foreach (element_children($form) as $key) {
    $row = array();
    foreach ($form[$key] as $item => $data) {
      if($item == 'label' || is_numeric($item))
        $row[] = form_render($data);
    }
    $rows[] = $row;
  }
  $header = $form['header']['#data'];
  return theme_table($header, $rows);
}

/**
 * Implementation of theme_reptag_exclude().
 *  - theming for reptag_exclude
 */
function theme_reptag_exclude($form) {
  foreach (element_children($form) as $key) {
    $row = array();
    $row[] = form_render($form[$key]['role']);
    $row[] = form_render($form[$key]['tags']);
    $row[] = form_render($form[$key]['add']);
    $row[] = form_render($form[$key]['remove']);
    $rows[] = $row;
  }
  $header = array(t('Exclude for role'), t('Tags'), '', '');
  return theme_table($header, $rows);
}

/**
 * Implementation of theme_reptag_exclude_add().
 *  - theming for reptag_exclude_add
 */
function theme_reptag_exclude_add($form) {
  $row = array();
  if (isset($form['value'])) {
    $row[] = form_render($form['value']);
    $row[] = form_render($form['button']);
  }
  $rows[] = $row;
  $header = array(t('Enter tag to exclude:'), '');
  $prefix = '<br /><div id="edit-roles-exclude-container-add-div" style="display:none">';
  $suffix = '</div><div id="roles-exclude-container"></div><input name="roles-exclude-rid" id="roles-exclude-rid" type="hidden" value="" />';
  return $prefix.theme_table($header, $rows).$suffix;
}

function theme_reptag_button($element) {
  return '<input type="button" class="form-'. $element['#button_type'] .'" name="'. $element['#name'] .'" value="'. check_plain($element['#value']) .'" '. drupal_attributes($element['#attributes']) ." />\n";
}

?>
