<?php

/**
 * Implementation of _reptag_maintenance().
 *  - maintenance options, mainly related to partial cache -
 */
function _reptag_maintenance() {

  $stored = db_num_rows(db_query("SELECT * FROM {reptag_storage}"));

  $form['maintenance'] = array(
    '#type' => 'fieldset',
    '#title' => t('RepTag - Maintenance'),
    '#collapsible' => FALSE,
    '#collapsed' => FALSE
  ); 
  $form['maintenance']['reptag_maintenance_migrate'] = array(
    '#type' => 'checkbox',
    '#title' => t('Migrate existing nodes to reptag storage'),
    '#return_value' => 1,
    '#default_value' => FALSE,
    '#description' => t('Use this option after you enabled \'PartialCache\' to create a cacheable version of every node.'),
    '#attributes' => ((variable_get('reptag_cache', 0) && !$stored) ? array() : array('disabled' => '' ))
  );
  $form['maintenance']['reptag_maintenance_regenerate'] = array(
    '#type' => 'checkbox',
    '#title' => t('Regenerate all cached pages'),
    '#return_value' => 1,
    '#default_value' => FALSE,
    '#description' => t('Use this option to update all cached pages. Reptag will run again on the original node and create a new cacheable version.'),
    '#attributes' => ((variable_get('reptag_cache', 0) && $stored) ? array() : array('disabled' => '' ))
  );
  $form['maintenance']['reptag_maintenance_restore'] = array(
    '#type' => 'checkbox',
    '#title' => t('Restore nodes from reptag storage'),
    '#return_value' => 1,
    '#default_value' => FALSE,
    '#description' => t('Use this option after you disabled \'PartialCache\' to remove the cached pages and restore the original nodes.'),
    '#attributes' => ($stored ? array() : array('disabled' => '' ))
  );
  $form['maintenance']['interval'] = array('#theme' => 'reptag_maintenance_interval');
  $form['maintenance']['interval']['pretitle'] = array('#value' => t('Process'));
  $form['maintenance']['interval']['posttitle'] = array('#value' => t('per step'));
  $form['maintenance']['interval']['reptag_interval'] = array(
    '#type' => 'select',
    '#title' => '',
    '#default_value' => variable_get('reptag_interval', 20),
    '#options' => array(
      20 => t('%n nodes', array('%n' => 20)),
      50 => t('%n nodes', array('%n' => 50)),
      100 => t('%n nodes', array('%n' => 100)),
      500 => t('%n nodes', array('%n' => 500)),
      1000 => t('%n nodes', array('%n' => 1000)),
      5000 => t('%n nodes', array('%n' => 5000))),
    '#attributes' => (variable_get('reptag_cache', 0) ? array() : array('disabled' => '' ))
  );
  $form['maintenance']['info'] = array(
    '#value' => t('Warning: Be careful using these options. There will be no backup of your database tables.'),
    '#prefix' => '<p><strong>',
    '#suffix' => '</strong></p>'
  );
  
  $form['buttons']['submit'] = array('#type' => 'submit', '#value' => t('Perform'));
  return drupal_get_form('_reptag_maintenance', $form);
}

function theme_reptag_maintenance_interval($form) {

  $rows[] = array(
    form_render($form['pretitle']),
    form_render($form['reptag_interval']),
    form_render($form['posttitle'])
  );
  
  return theme_table(array(), $rows);
}

/**
 * Implementation of _reptag_maintenance_submit().
 * - for _reptag_maintenance -
 */
function _reptag_maintenance_submit($form_id, $form_data) {

  if ($_POST['op'] == t('Perform')) {
    
    $interval = $form_data['reptag_interval'];
    variable_set('reptag_interval', is_numeric($interval) ? $interval : variable_get('reptag_interval', 20));
  
    if (($form_data['reptag_maintenance_migrate']
      + $form_data['reptag_maintenance_regenerate']
      + $form_data['reptag_maintenance_restore']) > 1
      ) {
      drupal_set_message(t('More than one option selected.'), 'error');
      return;
    }
    
    if ($form_data['reptag_maintenance_migrate']) {
      _reptag_maintenance_start('migrate');
    } elseif ($form_data['reptag_maintenance_regenerate']) {
      _reptag_maintenance_start('regenerate');
    } elseif ($form_data['reptag_maintenance_restore']) {
      _reptag_maintenance_start('restore');
    } else {
      drupal_set_message(t('No option selected.'), 'error');
    }
  }
}

function _reptag_maintenance_migrate(&$position) {
  require_once (drupal_get_path('module', 'reptag').'/reptag_process.inc');
  
  $result = db_query("SELECT * FROM {node_revisions}");
  $total = db_num_rows($result);
  if (!$total) {
    return 100;
  }
  
  $interval = variable_get('reptag_interval', 20);
  $result = db_query_range("SELECT * FROM {node_revisions}", $position, $interval);
  while ($node = db_fetch_object($result)) {
    //debug
    if (variable_get('reptag_debug', 0)) {
      drupal_set_message('migrate: nid '.$node->nid.', vid '.$node->vid, 'error');
    }
    _reptag_storage_setpage($node);
    reptag_process($node, REPTAG_NODETEASER, REPTAG_PROCCACHE);
    reptag_process($node, REPTAG_NODEFULL, REPTAG_PROCCACHE);
    _reptag_node_update($node);
  }
  $position += $interval;
  
  return ( 100 * $position ) / $total;
}

function _reptag_maintenance_regenerate(&$position) {
  require_once (drupal_get_path('module', 'reptag').'/reptag_process.inc');
  
  $result = db_query("SELECT * FROM {reptag_storage}");
  $total = db_num_rows($result);
  if (!$total) {
    return 100;
  }
  
  $interval = variable_get('reptag_interval', 20);
  $result = db_query_range("SELECT * FROM {reptag_storage}", $position, $interval);
  while ($node = db_fetch_object($result)) {
    if (variable_get('reptag_debug', 0)) {
      drupal_set_message('regenerate: nid '.$node->nid.', vid '.$node->vid, 'error');
    }
    reptag_process($node, REPTAG_NODETEASER, REPTAG_PROCCACHE);
    reptag_process($node, REPTAG_NODEFULL, REPTAG_PROCCACHE);
    _reptag_node_update($node);
  }
  $position += $interval;
  
  return ( 100 * $position ) / $total;
}

function _reptag_maintenance_restore(&$position) {
  require_once (drupal_get_path('module', 'reptag').'/regtag_helper.inc');
  
  $result = db_query("SELECT * FROM {reptag_storage}");
  $total = db_num_rows($result);
  if (!$total) {
    return 100;
  }
  
  $interval = variable_get('reptag_interval', 20);
  $result = db_query_range("SELECT * FROM {reptag_storage}", $position, $interval);
  while ($node = db_fetch_object($result)) {
    if (variable_get('reptag_debug', 0)) {
      drupal_set_message('restore: nid '.$node->nid.', vid '.$node->vid, 'error');
    }
    _reptag_node_update($node);
  }
  $position += $interval;
  if ($position >= $total) {
    db_query("TRUNCATE TABLE {reptag_storage}");
  }
  
  return ( 100 * $position ) / $total;
}

function _reptag_maintenance_start($mode) {
  $_SESSION['reptag_progress_mode'] = $mode;
  $_SESSION['reptag_progress_position'] = 0;
  drupal_goto('admin/settings/reptag/maintenance_progress');
}

function _reptag_maintenance_progress() {
  drupal_set_title('Rep[lacement]Tags Maintenance');
  $mode = $_SESSION['reptag_progress_mode'];
  $position = $_SESSION['reptag_progress_position'];
  if ($_SERVER['REQUEST_METHOD'] == 'GET') {
    switch ($mode) {
      case 'migrate':
        $msg = t('Migrating existing nodes ...');
        $percent = _reptag_maintenance_migrate($position);
        break;
      case 'regenerate':
        $msg = t('Regenerating cached pages ...');
        $percent = _reptag_maintenance_regenerate($position);
        break;
      case 'restore':
        $msg = t('Restoring node revisions ...');
        $percent = _reptag_maintenance_restore($position);
        break;
      default:
        $msg = t('Running maintenance ...');
        (int)fmod($position, 100.0);
    }
    if ($percent >= 100) {
      unset($_SESSION['reptag_progress_mode']);
      unset($_SESSION['reptag_progress_position']);
      drupal_set_message(t('Maintenance operation successfully comleted.'));
      drupal_goto('admin/settings/reptag/maintenance');
      return NULL;
    }
    $_SESSION['reptag_progress_position'] = $position;
  }

  drupal_set_html_head('<meta http-equiv="Refresh" content="0; URL=index.php?q=admin/settings/reptag/maintenance_progress">');
  print theme('maintenance_page', '<div style="width:50%; height:45px;">'.theme('progress_bar', $percent, $msg).'</div>', FALSE, TRUE);
  return NULL;
}

?>
